
!* ---------------------------------------------------------------------- LOCUT
      subroutine locut(s,nd,fcut,delt,nroll,icaus)

c  Butterworth locut filter order 2*nroll (nroll<=8) (see Kanasewich, 
c    Time Sequence Analysis in Geophysics, Third Edition, 
c    University of Alberta Press, 1981)
c  written by W. B. Joyner 01/07/97
c  causal if icaus.eq.1 - zero phase shift otherwise
c  s(j) input = the time series to be filtered - output = the 
c    filtered series
c  dimension of s(j) must be at least as large as 
c    nd+3.0*float(nroll)/(fcut*delt)
c  nd = the number of points in the time series
c  fcut = the cutoff frequency
c  delt = the timestep

* There is a fundamental difference in the shapes of a causal and acausal 
* filter.

* The response for a causal filter is given by eq. 15.8-6 in Kanasewich:
*    Y = sqrt((f/fcut)**(2*n)/(1+(f/fcut)**(2*n))), where n = 2*nroll
* The acausal filter is obtained by applying the causal filter twice.
* The consequence is that the response of the acausal filter is
*    Y =      (f/fcut)**(2*n)/(1+(f/fcut)**(2*n)),  where n = 2*nroll
* Note that for the same nroll as input, for f/fcut << 1
* the causal filter goes as
*    Y -> (f/fcut)**(2*nroll)
* whereas the acausal filter falls off more rapidly:
*    Y -> (f/fcut)**(4*nroll)
* Also note the difference when f = fcut:
*   causal:  Y = 1/sqrt(2)
*  acausal:  Y = 1/2
* This is true no matter what value is chosen for nroll.  Because of this,
* it is not possible to make the response of the causal and acausal filters the
* same.  In particular, caution should be used if nroll is chosen so as to
* make the low-frequency asymptotes the same.  This requires nroll for the 
* acausal filter to be half that of the causal filter.  The problem is that
* the response for frequencies higher than fcut will be reduced for the
* acausal as compared to the causal filter.  


* Dates: xx/xx/xx - Written by Bill Joyner
*        09/30/97 - double precision statements added so that the output
*                   series has sufficient precision for double integration.
*        12/17/99 - D. Boore added check for fcut = 0.0, in which case
*                   no filter is applied.  He also cleaned up the
*                   appearance of the code (indented statements in
*                   loops, etc.)
*        02/04/00 - Changed "n" to "nroll" to eliminate confusion with
*                   Kanesewich, who uses "n" as the order (=2*nroll)
*        09/12/00 - Made trig functions double precision
*        11/08/00 - Increased dimension of s from 50000 to 100000
*        02/14/01 - Modified/corrected real numbers function calls to 
*                   double precision - cds
*        02/15/01 - Added response of acausal filter in the comments
*        02/22/01 - Removed dimension of s (it is up to the user to specify
*                   it properly)
*        02/22/01 - Added more discussion about the shapes of the causal
*                   and acausal filters.
*        07/20/01 - No filter of fcut .le. 0.0
!        08/01/12 - Added "dble" to b1(k)= statement and changed "2.0d0" to "2d0"
!                   (before doing that, part of the statement was highlighted in
!                   red when viewed in Lahey's ED4W, although I don't know why).

      real s(*)
      real*8 fact(8),b1(8),b2(8)
      real*8 pi,w1,xp,yp,x1,x2,y1,y2


      if (fcut .le. 0.0) return       ! Added by DMB

      pi=4.0d0*datan(1.0d0)
      w1=2.0d0*pi*fcut
      w1=2.0d0*dtan(w1*delt/2.0d0)/delt

      do k=1,nroll
        fact(k)=1.0d0/(1.0d0+dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))
     *   *w1*delt+
     *   (w1*delt/2.0d0)**2)
        b1(k)=( (w1*dble(delt))**2/2d0-2d0 )*fact(k)
        b2(k)=(1.0d0-dsin(pi*dfloat(2*k-1)/dfloat(4*nroll))*w1*delt+
     *   (w1*delt/2.0d0)**2)*fact(k)
      end do

      np2=nd

      if(icaus.ne.1) then
        npad=3.0*float(nroll)/(fcut*delt)
        np1=nd+1
        np2=nd+npad
        do j=np1,np2
          s(j)=0.0
        end do
      end if

      do k=1,nroll
        x1=0.0d0
        x2=0.0d0
        y1=0.0d0
        y2=0.0d0
        do j=1,np2
          xp=s(j)
          yp=fact(k)*(xp-2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
          s(j)=yp
          y2=y1
          y1=yp
          x2=x1
          x1=xp
        end do
      end do

      if(icaus.ne.1) then
        do k=1,nroll
          x1=0.0d0
          x2=0.0d0
          y1=0.0d0
          y2=0.0d0
          do j=1,np2
            xp=s(np2-j+1)
            yp=fact(k)*(xp-2.0d0*x1+x2)-b1(k)*y1-b2(k)*y2
            s(np2-j+1)=yp
            y2=y1
            y1=yp
            x2=x1
            x1=xp
          end do
        end do
      end if

      return
      end
!* ---------------------------------------------------------------------- LOCUT
