! --------------------------- BEGIN DIST_3DF ---------------------
      subroutine dist_3df(cartesian_coordinates, N_sta, E_sta, 
     :   N_ref, E_ref, h_ref, strike_f, dip_f, w1, w2, s1, s2, 
     :   h_min_c, 
     :   d_jb, az_jb, rx_nga, ry0_nga,
     :   d_cd2f, az_cd2f, d_c, az_c,
     :   d_sta_n, d_sta_e, icase_cd2f, icase_c, icase_jb)

! Computes various distance measures from a station to a fault.  The 
! orientation of the fault is that used by Spudich et al., Yucca Mt. project.
! The fault is assumed to be a rectangle whose upper and lower edges are
! horizontal.

! NOTE *******
! There is a confusing mix of notations for axis orientations. For example, 
! the along strike and downdip unit vectors of the fault are x and z, respectively
! because for sone strange reason this is what I chose when deriving the rotation
! equations).  But rx_nga and ry0_nga refer to distances perpendicular anbd along strike.
! Because the program works, I decided not to rename the variables.
! NOTE *******

! Input:
!      N_sta, E_sta:  latitude and longitude of station, in degrees,
!                            west longitude is negative
!      N_ref, E_ref:  as above, for reference point used in defining
!                            fault orientation (it must be in the plane of the fault)
!      h_ref:                depth to reference point
!      strike_f, dip_f:      azimuth and dip of fault, in degrees.  strike_f is
!                            measured positive clockwise from north; dip_f is
!                            measured from the horizontal.  When looking in 
!                            the direction strike_f, a positive dip is down to
!                            the right.
!      w1, w2, s1, s2:       distances from the reference point to the edges of
!                            the fault.  s1 and s2 are the distances along
!                            strike to the near and far edges; w1 and w2 are
!                            the distances along the dip direction to the upper
!                            and lower edges of the fault.
!      h_min_c:              minimum depth for computing Campbell's distance
!                            (usually 3.0 km)

! Output:
!      d_jb, d_cd2f, d_c:    Joyner & Boore, closest distance to fault surface,
!                            and Campbell distance, respectively.
!      az_jb, az_cd2f, az_c: as above, for azimuths
!      rx_nga:               horizontal distance perpendicular to fault strike 
!                            (positive on hanging wall side) 
!                            (used in some NGA-W2 GMPEs)
!      ry0_nga:              A distance along strike, used by Abrahamson et al. (2014)
!                            to taper their hanging wall effect.  It can only be greater
!                            than or equal to 0.0. See p. 1040 of Abrahamson et al. (2014), 
!                            EqSpectra 30, 1025-1055.
!      d_sta_n, d_sta_e:     north and east components of station location
!                            relative to the reference point
!      irgn_cd2f, etc:       region in fault-plane coordinates used to 
!                            compute distances.  I could include a sketch here,
!                            but I will not take the time.  These output 
!                            variables were included mainly to help me check
!                            the subroutine.




! Dates: 12/06/98 - Written by D. Boore
!        09/17/00 - Bring in subroutines via include statement;
!                   renamed faz and az_f to fstrike, strike_f
!        11/12/01 - Changed specification in headers above to indicate that
!                   west longitude is negative (consistent with revision to
!                   subroutine deg2km_f)
!        09/16/10 - Compute az_jb (following Kaklamanos et al definitions).
!                 - Changed "rgn" to "case" (Kaklamanos et al., terminology and definition).
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)
!        01/10/17 - Add rx_nga to the output
!        12/17/19 - Add ry0_nga to the output
!                 - Add some comments about nomenclature for coordinate systems
!        12/20/19 - Allow reference and station coordinates to be in Cartesian 
!                   coordinates or geographic coordinates (lat/long)

      implicit none

      real d_sta_e, d_sta_n, az_c, d_c, az_cd2f, d_cd2f, ry0_nga,
     :  rx_nga, az_jb, d_jb, h_min_c, s2, s1, w2, w1, dip_f, strike_f,
     :  h_ref, E_ref, N_ref, E_sta, N_sta, pi, dtor, fstrike, fdip,  
     :  rx, ry, rz, h_cd2f, d2top_c, d2top, w1_c, h_c, s1_jb, s2_jb, 
     :  w1_jb, w2_jb, rx_jb, rz_jb, h_jb

      integer icase_jb, icase_c, icase_cd2f, i

      logical cartesian_coordinates

      real dist_sta(3)
      real ix(3), iy(3), iz(3)

      pi = 4.0*atan(1.0)
      dtor = pi/ 180.0

! set up unit vectors in fault coordinates in terms of north, east, down 
! unit vectors.

! Convert angles to radians:
      fstrike =  dtor * strike_f
      fdip = dtor * dip_f

! Initialize arrays:

      do i = 1, 3
         ix(i) = 0.0
         iy(i) = 0.0
         iz(i) = 0.0
      end do

! Compute unit vectors:        ! 1, 2, 3 correspond to n, e, d

      ix(1) = cos(fstrike)
      ix(2) = sin(fstrike)
      ix(3) = 0.0

      iy(1) = -sin(fstrike)*sin(fdip)
      iy(2) =  cos(fstrike)*sin(fdip)
      iy(3) =          -cos(fdip)

      iz(1) = -sin(fstrike)*cos(fdip)
      iz(2) =  cos(fstrike)*cos(fdip)
      iz(3) =           sin(fdip)

! Compute cartesian distance from the reference point to the station
      if(cartesian_coordinates) then
        dist_sta(1) = N_sta - N_ref
        dist_sta(2) = E_sta - E_ref
      else
! Convert station lat, long into distance north and east:
        call deg2km_f(N_sta, E_sta, N_ref, E_ref,
     :              dist_sta(1), dist_sta(2))
      end if
      dist_sta(3) = -h_ref    ! note minus sign
      
      d_sta_n = dist_sta(1)
      d_sta_e = dist_sta(2)

! Convert coordinates of reference-to-station vector from n,e,d coordinates
! into fault coordinates:
 
      rx = 0.0
      ry = 0.0
      rz = 0.0
  
      do i = 1, 3
        rx = rx + dist_sta(i) * ix(i)
        ry = ry + dist_sta(i) * iy(i)
        rz = rz + dist_sta(i) * iz(i)
      end do

! Find region and closest distance to fault in the fault plane coordinates:

      call find_h(rx, rz, w1, w2, s1, s2, h_cd2f,
     :            icase_cd2f)                         ! cd2f = Closest Distance
                                                      !        to Fault

! Now do it for Campbell:

! Define w1 for Campbell (I assume that w2 does not need defining; in other
! words, not all of the fault plane is above the Campbell depth)

      d2top_c = h_min_c
      d2top = h_ref + w1 * iz(3)        ! iz(3) = sin(fdip)
      if ( d2top .lt. d2top_c .and. iz(3) .ne. 0.0) then
        w1_c = (d2top_c - h_ref)/ iz(3)
      else
        w1_c = w1
      end if

      call find_h(rx, rz, w1_c, w2, s1, s2, h_c,      ! c = Campbell
     :            icase_c)

! (Work on azimuths later)
      az_cd2f = 999.9
      az_c    = 999.9

! Now do it for Joyner-Boore:

! Need to find rx, ry, rz, w1, w2, s1, s2 in terms of coordinates
! of the fault plane projected onto the surface:

      s1_jb = s1
      s2_jb = s2
      w1_jb = w1 * cos(fdip)
      w2_jb = w2 * cos(fdip)

      rx_jb = rx
      rz_jb = -sin(fstrike) * dist_sta(1) + cos(fstrike) * dist_sta(2)
      
      rx_nga = rz_jb - w1_jb

! Then find the region and distance in the plane to the fault surface
      call find_h(rx_jb, rz_jb, 
     :            w1_jb, w2_jb, s1_jb, s2_jb, h_jb,
     :            icase_jb)

! Now compute the distances:

      d_cd2f = sqrt(h_cd2f**2 + ry   **2)
      d_c    = sqrt(h_c   **2 + ry   **2)
      d_jb   = h_jb

      if (icase_jb == 1 .or. icase_jb == 2 .or. icase_jb == 3 ) then
        az_jb = atan2(rz_jb - w1_jb, rx_jb - s2_jb)/dtor
      else if (icase_jb == 4) then
        az_jb = -90.0
      else if (icase_jb == 5 .or. icase_jb == 6) then
        az_jb = +90.0
      else if (icase_jb == 7 .or. icase_jb == 8 .or. icase_jb == 9 )
     :                                                             then
        az_jb = atan2(rz_jb - w1_jb, rx_jb - s1_jb)/dtor
      else
        write(*,*) 
     :   ' Computing az_jb in dist_3df, icase_jb /= 1-9; QUIT!'
        stop        
      end if
      
! Compute ry0_nga

      ry0_NGA = 0.0
      if (icase_jb == 2 .or. icase_jb == 3 ) then
        ry0_nga = rx_jb - s2_jb
      else if (icase_jb == 8 .or. icase_jb == 9 ) then
        ry0_nga = abs(rx_jb - s1_jb)
      else
        ry0_nga = 0.0
      end if
      
!DEBUG
      write(*,'(a)') ' In dist_3df, icase_jb, rx_jb, s2_jb, s1_jb, '// 
     : 'ry0_nga, rx_nga = '
      write(*, *) icase_jb, rx_jb, s2_jb, s1_jb, 
     :  ry0_nga, rx_nga
!DEBUG
      
      return

      end
! --------------------------- END DIST_3DF ---------------------

!-------------------- BEGIN FIND_H ----------------------------
      subroutine find_h(rx, rz, w1, w2, s1, s2, h, icase)

! 09/16/10 - Redefine regions to be those of Kaklamanos et al
!        04/28/15 - Replaced comment characters * or C with ! (The Fortran 95 standard)

! Here is a conversion table:

!    My notes (old definition of region)   Kaklamanos et al case
!           1                                     7
!           2                                     4
!           3                                     1
!           4                                     2
!           5                                     3
!           6                                     6
!           7                                     9
!           8                                     8
!           9                                     5

! Now it is easy to see where the station lies with respect to the fault;
! there are 9 possibilities:  

      implicit none

      integer icase
      
      real h, s2, s1, w2, w1, rz, rx
      
      if (   rx .le. s1 .and. rz .le. w1 ) then
! old region 1 (see notes)
!        iregion = 1
        icase = 7
        h = sqrt( (s1-rx)**2 + (w1-rz)**2 )
       
      else if (   rz .le. w1 .and. rx .ge. s1 .and. rx .le. s2 ) then
! old region 2 (see notes)
!        iregion = 2
        icase = 4
        h = w1 - rz

      else if (   rx .ge. s2 .and. rz .le. w1 ) then
! old region 3 (see notes)
!        iregion = 3
        icase = 1
        h = sqrt( (rx-s2)**2 + (w1-rz)**2 )
        
      else if (   rx .ge. s2 .and. rz .ge. w1 .and. rz .le. w2 ) then
! old region 4 (see notes)
!        iregion = 4
        icase = 2
        h = rx - s2

      else if (   rx .ge. s2 .and. rz .ge. w2 ) then
! old region 5 (see notes)
!        iregion = 5
        icase = 3
        h = sqrt( (rx-s2)**2 + (rz-w2)**2 )
        
      else if (   rz .ge. w2 .and. rx .ge. s1 .and. rx .le. s2 ) then
! old region 6 (see notes)
!        iregion = 6
        icase = 6
        h = rz - w2

      else if (   rz .ge. w2 .and. rx .le. s1 ) then
! old region 7 (see notes)
!        iregion = 7
        icase = 9
        h = sqrt( (s1-rx)**2 + (rz-w2)**2 )
        
      else if (   rx .le. s1 .and. rz .ge. w1 .and. rz .le. w2 ) then
! old region 8 (see notes)
!        iregion = 8
        icase = 8
        h = s1 - rx

      else if (      rx .ge. s1 .and. rx .le. s2 
     :         .and. rz .ge. w1 .and. rz .le. w2 ) then
! old region 9 (see notes)
!        iregion = 9
        icase = 5
        h = 0.0

      else
! reaching this is an error
        write(*,'(a)') ' ERROR: Region not found in find_h'

      end if

      return
      end
!-------------------- END FIND_H ----------------------------

!      include '\forprogs\deg2km_f.for'
!      include '\forprogs\locate.for'

