*----------------- BEGIN V2A -----------------------------
      subroutine v2a(vel, npts_v, dt, acc, npts_a, itype)


* Compute acceleration from velocity, using the reverse process
* of going from acceleration to velocity,
* assuming that the acceleration
* is represented by straight lines connecting the digitized values.

* itype = -99: use the reverse of going from acceleration to velocity 
*            assuming lines connecting acceleration points.
* itype = 3: centered three point derivative
* itype = 5: centered five point derivative

* For consistency, I assume the vel(1) and acc(1) = 0.0, even if that is not
* true of vel(1).

* Dealing with end points for the itype=3 and 5 operators:  Strictly speaking, the values of 
* acc(1), acc(2), acc(npts-1), and acc(npts) are not defined.  I could
* assume that vel(0)=vel(-1)=vel(npts+1)=vel(npts+2)=0 and compute the
* derivative at i=1,2,npts-1,npts using the formula, I could set those values
* to 0.0, or I could trim the acc array to include values from i=3...npts-2.
* Until 15 June 2009 I assumed 0.0 for the vel outside of the input range, so that
* the output array will have the same size as the input array.  But on that date I
* adopted the SAC procedure for the missing values by simply reducing the number of points in the
* acceleration time series by the number of undefined points (and increasing the start
* time accordingly).

* But note this comment in acc2vd.for:
*        01/07/00 - Added initial velocity and displacements (v0, d0).
*                   This also requires the addition of a linear trend in
*                   displacement.
*                   Also, Bill Joyner, Chris Stephens, and I considered how to
*                   handle the first point.  Assuming v0 = 0, BAP uses a 
*                   trapezoidal rule such that the first vel(1) = 0.5*dt*a(1),
*                   but in acc2vd, vel(1) = 0.  In effect, BAP starts the
*                   integration at -dt rather than 0.0, with the result that
*                   because vel(1) depends on dt, vel(1) will change if the
*                   digitization interval is changed.  We agreed that this is 
*                   not correct.

* Dates: 08/11/02 - Written by D.M. Boore, patterned after acc2vd.for.
*        04/29/09 - Add type of derivative to use
*                   The original v2a seems to work very poorly, so eliminate it as a candidate.
*        06/13/09 - Let itype_diff_op equal the number of points in the differentiation operator.
*        06/15/09 - Adopt the SAC procedure of accounting for undefined
*                   acceleration values by reducing the number of points.

      integer itype, npts_v, npts_a
      real acc(*), vel(*)
      double precision v1, v2, v3, vm2, vm1, vp1, vp2, ddt, ddt_2,
     :                 f1, f2

      do i = 1, npts_v
        acc(i) = 0.0
      end do
      
      ddt = dble(dt)
        
      if (itype .eq. -99) then
      
* compute acceleration, using analytical formulas based
* on representing the acceleration as a series of straightline segments.


        ddt_2   = ddt/2.0d0

!      acc(1) = a0
        acc(1) = 0.0 

! Do the following so do not actually alter input array vel
        v2 = dble(vel(2))
        acc(2) = sngl((v2)/ddt_2) - acc(1)

        do j=3,npts_v
          v1 = dble(vel(j-1))
          v2 = dble(vel(j))
          acc(j) = sngl((v2-v1)/ddt_2) - acc(j-1)
        end do
        
        npts_a = npts_v
        
      else if (itype .eq. 3) then  ! "three point" operator

        npts_a = npts_v - 2
        do i=2,npts_v-1	  
          vp1 = dble(vel(i+1))
          vm1 = dble(vel(i-1))
	  acc(i-1) = sngl( (vp1 - vm1)/(2.0d0*ddt) )  
        end do
!        i=1
!        vp1 = dble(vel(i+1))
!	acc(1) = sngl(vp1/(2.0d0*ddt) )
!	i=npts
!        vm1 = dble(vel(i-1))
!	acc(npts) = sngl(-vm1/(2.0d0*ddt) )  

      else if (itype .eq. 5) then  ! "five point" operator
      
        f1=1.0d0/12.0d0
        f2=2.0d0/3.0d0
        
        npts_a = npts_v - 4
        do i=3,npts_v-2
          vm2 = dble(vel(i-2))
          vm1 = dble(vel(i-1))
          vp1 = dble(vel(i+1))
          vp2 = dble(vel(i+2))
          acc(i-2)=sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
        end do

!        i = 1
!          vm2 = 0.0d0
!          vm1 = 0.0d0
!          vp1 = dble(vel(i+1))
!          vp2 = dble(vel(i+2))
!          acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
!        i = 2
!          vm2 = 0.0d0
!          vm1 = dble(vel(i-1))
!          vp1 = dble(vel(i+1))
!          vp2 = dble(vel(i+2))
!          acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
!        i = npts-1
!          vm2 = dble(vel(i-2))
!          vm1 = dble(vel(i-1))
!          vp1 = dble(vel(i+1))
!          vp2 = 0.0d0
!          acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
!        i = npts
!          vm2 = dble(vel(i-2))
!          vm1 = dble(vel(i-1))
!          vp1 = 0.0d0
!          vp2 = 0.0d0
!          acc(i)= sngl((f1*vm2-f2*vm1+f2*vp1-f1*vp2)/ddt)
      
      else
      
        print *,' invalid itype = ', itype, '; QUITTING!!!'
        stop
        
      end if

      return
      end
*----------------- END V2A -----------------------------
